@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 # This file is part of urlwatch (https://thp.io/2008/urlwatch/).
-# Copyright (c) 2008-2016 Thomas Perl <thp.io/about>
+# Copyright (c) 2008-2018 Thomas Perl <m@thp.io>
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -109,8 +109,8 @@ class JobBase(object, metaclass=TrackSubClasses):
         if 'kind' not in data:
             # Try to auto-detect the kind of job based on the available keys
             kinds = [subclass.__kind__ for subclass in list(cls.__subclasses__.values())
-                     if all(required in data for required in subclass.__required__) and
-                     not any(key not in subclass.__required__ and key not in subclass.__optional__ for key in data)]
+                     if all(required in data for required in subclass.__required__) and not any(
+                     key not in subclass.__required__ and key not in subclass.__optional__ for key in data)]
 
             if len(kinds) == 1:
                 kind = kinds[0]
@@ -146,7 +146,7 @@ class JobBase(object, metaclass=TrackSubClasses):
 
 class Job(JobBase):
     __required__ = ()
-    __optional__ = ('name', 'filter')
+    __optional__ = ('name', 'filter', 'max_tries')
 
     def pretty_name(self):
         return self.name if self.name else self.get_location()
@@ -179,7 +179,8 @@ class UrlJob(Job):
     __kind__ = 'url'
 
     __required__ = ('url',)
-    __optional__ = ('cookies', 'data', 'method', 'ssl_no_verify', 'ignore_cached', 'http_proxy', 'https_proxy')
+    __optional__ = ('cookies', 'data', 'method', 'ssl_no_verify', 'ignore_cached', 'http_proxy', 'https_proxy',
+                    'headers')
 
     CHARSET_RE = re.compile('text/(html|plain); charset=([^;]*)')
 
@@ -221,6 +222,9 @@ class UrlJob(Job):
             logger.info('Using local filesystem (%s URI scheme)', file_scheme)
             return open(self.url[len(file_scheme):], 'rt').read()
 
+        if self.headers:
+            self.add_custom_headers(headers)
+
         response = requests.request(url=self.url,
                                     data=self.data,
                                     headers=headers,
@@ -252,3 +256,30 @@ class UrlJob(Job):
                 return response.content.decode('ascii', 'ignore')
 
         return response.text
+
+    def add_custom_headers(self, headers):
+        """
+        Adds custom request headers from the job list (URLs) to the pre-filled dictionary `headers`.
+        Pre-filled values of conflicting header keys (case-insensitive) are overwritten by custom value.
+        """
+        headers_to_remove = [x for x in headers if x.lower() in [y.lower() for y in self.headers]]
+        for header in headers_to_remove:
+            headers.pop(header, None)
+        headers.update(self.headers)
+
+
+class BrowserJob(Job):
+    """Retrieve an URL, emulating a real web browser"""
+
+    __kind__ = 'browser'
+
+    __required__ = ('navigate',)
+
+    def get_location(self):
+        return self.navigate
+
+    def retrieve(self, job_state):
+        from requests_html import HTMLSession
+        session = HTMLSession()
+        response = session.get(self.navigate)
+        return response.html.html
