@@ -1,6 +1,6 @@
 #
 # This file is part of urlwatch (https://thp.io/2008/urlwatch/).
-# Copyright (c) 2008-2016 Thomas Perl <thp.io/about>
+# Copyright (c) 2008-2018 Thomas Perl <m@thp.io>
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -343,19 +343,14 @@ class EMailReporter(TextReporter):
             logger.debug('Not sending e-mail (no changes)')
             return
         if self.config['method'] == "smtp":
-            if self.config['smtp']['user']:
-                self.smtp_user = self.config['smtp']['user']
-            else:
-                self.smtp_user = self.config['from']
-            mailer = SMTPMailer(self.smtp_user, self.config['smtp']['host'], self.config['smtp']['port'],
+            smtp_user = self.config['smtp'].get('user', self.config['from'])
+            mailer = SMTPMailer(smtp_user, self.config['smtp']['host'], self.config['smtp']['port'],
                                 self.config['smtp']['starttls'], self.config['smtp']['keyring'])
         elif self.config['method'] == "sendmail":
             mailer = SendmailMailer(self.config['sendmail']['path'])
         else:
             logger.error('Invalid entry for method {method}'.format(method=self.config['method']))
 
-        # TODO set_password(options.email_smtp, options.email_from)
-
         if self.config['html']:
             body_html = '\n'.join(self.convert(HtmlReporter).submit())
 
@@ -405,7 +400,9 @@ class PushoverReport(WebServiceReporter):
         return app.get_user(self.config['user'])
 
     def web_service_submit(self, service, title, body):
-        msg = service.create_message(title=title, message=body, html=True, sound='spacealarm')
+        sound = self.config['sound']
+        device = self.config['device']
+        msg = service.create_message(title=title, message=body, html=True, sound=sound, device=device)
         msg.send()
 
 
