@@ -35,15 +35,12 @@ import platform
 from abc import ABCMeta, abstractmethod
 
 import shutil
-
-import subprocess
-import shlex
 import yaml
 import json
 import minidb
 import logging
 
-from .util import atomic_rename
+from .util import atomic_rename, edit_file
 from .jobs import JobBase, UrlJob, ShellJob
 
 logger = logging.getLogger(__name__)
@@ -164,14 +161,6 @@ class BaseTextualFileStorage(BaseFileStorage, metaclass=ABCMeta):
         ...
 
     def edit(self, example_file=None):
-
-        editor = os.environ.get('EDITOR', None)
-        if editor is None:
-            editor = os.environ.get('VISUAL', None)
-        if editor is None:
-            print('Please set $VISUAL or $EDITOR.')
-            return 1
-
         fn_base, fn_ext = os.path.splitext(self.filename)
         file_edit = fn_base + '.edit' + fn_ext
 
@@ -182,13 +171,13 @@ class BaseTextualFileStorage(BaseFileStorage, metaclass=ABCMeta):
 
         while True:
             try:
-                editor = shlex.split(editor)
-                editor.append(file_edit)
-                subprocess.check_call(editor)
+                edit_file(file_edit)
                 # Check if we can still parse it
                 if self.parse is not None:
                     self.parse(file_edit)
                 break  # stop if no exception on parser
+            except SystemExit:
+                raise
             except Exception as e:
                 print('Parsing failed:')
                 print('======')
@@ -370,7 +359,7 @@ class CacheStorage(BaseFileStorage, metaclass=ABCMeta):
         ...
 
     @abstractmethod
-    def save(self, job, guid, data, timestamp):
+    def save(self, job, guid, data, timestamp, tries):
         ...
 
     @abstractmethod
@@ -383,12 +372,12 @@ class CacheStorage(BaseFileStorage, metaclass=ABCMeta):
 
     def backup(self):
         for guid in self.get_guids():
-            data, timestamp = self.load(None, guid)
-            yield guid, data, timestamp
+            data, timestamp, tries = self.load(None, guid)
+            yield guid, data, timestamp, tries
 
     def restore(self, entries):
-        for guid, data, timestamp in entries:
-            self.save(None, guid, data, timestamp)
+        for guid, data, timestamp, tries in entries:
+            self.save(None, guid, data, timestamp, tries)
 
     def gc(self, known_guids):
         for guid in set(self.get_guids()) - set(known_guids):
@@ -453,6 +442,7 @@ class CacheEntry(minidb.Model):
     guid = str
     timestamp = int
     data = str
+    tries = int
 
 
 class CacheMiniDBStorage(CacheStorage):
@@ -474,15 +464,15 @@ class CacheMiniDBStorage(CacheStorage):
         return (guid for guid, in CacheEntry.query(self.db, minidb.Function('distinct', CacheEntry.c.guid)))
 
     def load(self, job, guid):
-        for data, timestamp in CacheEntry.query(self.db, CacheEntry.c.data // CacheEntry.c.timestamp,
-                                                order_by=CacheEntry.c.timestamp.desc,
-                                                where=CacheEntry.c.guid == guid, limit=1):
-            return data, timestamp
+        for data, timestamp, tries in CacheEntry.query(self.db, CacheEntry.c.data // CacheEntry.c.timestamp // CacheEntry.c.tries,
+                                                       order_by=minidb.columns(CacheEntry.c.timestamp.desc, CacheEntry.c.tries.desc),
+                                                       where=CacheEntry.c.guid == guid, limit=1):
+            return data, timestamp, tries
 
-        return None, None
+        return None, None, 0
 
-    def save(self, job, guid, data, timestamp):
-        self.db.save(CacheEntry(guid=guid, timestamp=timestamp, data=data))
+    def save(self, job, guid, data, timestamp, tries):
+        self.db.save(CacheEntry(guid=guid, timestamp=timestamp, data=data, tries=tries))
         self.db.commit()
 
     def delete(self, guid):
